@@ -15,7 +15,7 @@ from attrs import define
 
 __author__ = "Florian Matter"
 __email__ = "florianmatter@gmail.com"
-__version__ = "0.0.5"
+__version__ = "0.0.6"
 
 
 handler = colorlog.StreamHandler(None)
@@ -25,6 +25,7 @@ handler.setFormatter(
 log = logging.getLogger(__name__)
 log.propagate = True
 log.addHandler(handler)
+log.setLevel(logging.INFO)
 
 
 person_values = ["1", "2", "3", "1+3", "1+2"]
@@ -286,8 +287,12 @@ class Pyradigm:
             value_name="Value",
         )
 
-    def compose_paradigm(  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
-        self, csv_output=None, **kwargs
+    def compose_paradigm(  # pylint: disable=too-many-locals
+        # pylint: disable=too-many-branches
+        # pylint: disable=too-many-statements
+        self,
+        csv_output=None,
+        **kwargs,
     ):
         """The central function of pyradigms, creating paradigms. In addition to the list
         below, **you can pass any
@@ -311,6 +316,9 @@ class Pyradigm:
         print_column = kwargs.get("print_column", self.print_column)
         sort_orders = kwargs.get("sort_orders", self.sort_orders)
         output_folder = kwargs.get("output_folder", self.output_folder)
+        decorate_x = kwargs.get("decorate_x", lambda x: x)
+        decorate_y = kwargs.get("decorate_y", lambda y: y)
+        decorate = kwargs.get("decorate", lambda x: x)
         if output_folder:
             output_folder = Path(output_folder)
 
@@ -320,8 +328,9 @@ class Pyradigm:
 
         # get a sensible default sort order for a given parameter (order in the input)
         def get_sort_order(parameter):
-            val_list = list(df[parameter])
-            val_list = [i for n, i in enumerate(val_list) if i not in val_list[:n]]
+            if parameter in filters:
+                return filters[parameter]
+            val_list = list(dict.fromkeys(list(df[parameter])))
             log.debug(f"New sort order for {parameter}: {val_list}")
             return val_list
 
@@ -349,7 +358,7 @@ class Pyradigm:
             log.info(
                 "You did not specify what should happen"
                 " to the following columns/fields/parameters: %s",
-                "\t".join(leftover_columns),
+                ", ".join(leftover_columns),
             )
 
         # only for debugging purposes
@@ -358,7 +367,7 @@ class Pyradigm:
         )
         log.debug(f"Filtering parameters:\n{filter_string}\n")
 
-        # filter rows by fitler arg
+        # filter rows by filter arg
         for col, values in filters.items():
             values = _listify(values)
             df = df[df[col].isin(values)]
@@ -397,6 +406,7 @@ class Pyradigm:
             log.debug(
                 f"Creating pivot table for x={x}, y={y}, cell values: {print_column}"
             )
+            df[print_column] = df[print_column].map(decorate)
             out = pd.pivot_table(
                 df,
                 values=print_column,
@@ -421,11 +431,12 @@ class Pyradigm:
             for parameter in out.index.names + out.columns.names:
                 df_sort = get_sort_order(parameter)
                 if parameter not in sort_orders:
+                    log.info(f"Guessing order {df_sort} for parameter {parameter}")
                     sort_orders[parameter] = df_sort
                 elif set(sort_orders[parameter]) != set(df_sort):
                     log.error(
                         f"Specified order {sort_orders[parameter]} for parameter "
-                        f"'{parameter}' does not cover all values: {df_sort}"
+                        f"'{parameter}' does not cover all values: {df_sort}."
                     )
                     sys.exit(1)
 
@@ -456,21 +467,20 @@ class Pyradigm:
 
             if not with_multi_index:
                 log.debug("Flattening multiindices")
-
                 new_colindex_name = category_joiner.join(x)
                 out.columns = [
-                    _format_person_values(
+                    decorate_x(_format_person_values(
                         separators[0].join(col).strip(), separators[0]
-                    )
+                    ))
                     for col in out.columns.values
                 ]
                 out.columns.name = new_colindex_name
 
                 new_index_name = category_joiner.join(y)
                 out.index = [
-                    _format_person_values(
+                    decorate_y(_format_person_values(
                         separators[0].join(_listify(col)).strip(), separators[0]
-                    )
+                    ))
                     for col in out.index.values
                 ]
                 out.index.name = new_index_name
@@ -482,7 +492,7 @@ class Pyradigm:
             output = []
             for z_key, df in constructed_paradigms.items():
                 s = StringIO()
-                df.to_csv(s, index=True)
+                df.to_csv(s, index=True, index_label="")
                 output.append(s.getvalue())
             with open(csv_output, "w", encoding="utf-8") as file:
                 file.write("\n".join(output))
